; ArrowLISP Extended Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Convert a given date to the number of days
; since 01-01-01. This function will deliver
; wrong results for dates before 1582-10-15.

; ---args---
; DATE - date

; ---keywords---
; DATE-TO-DAYS function, date, time, absolute date, days.

; ---see-also---
; date, date-before-p, date-difference

; ---example---
; (date-to-days '(#2003 #9 #21))

:c lib/caar.l
:c lib/caaar.l
:c lib/logand.l
:c lib/zerop.l
:c lib/greaterp.l
:c lib/remainder.l
:c lib/difference.l
:c lib/plus.l
:c lib/times.l
:c lib/quotient.l
:c lib/nth.l
:c lib/incr.l
:c lib/decr.l
:c ext/date.l
;c ext/year-of.l
;c ext/month-of.l
;c ext/day-of.l

; ---code---
(defun date-to-days (date)
	(label

	((days-per-month
		'(#0 #31 #59 #90 #120 #151 #181 #212 #243 #273 #304 #334))

	(leap-year-p (lambda (year)
		(cond ((zerop (remainder year '#400)) *t*)
			((zerop (remainder year '#100)) *f*)
			(t (zerop (remainder year '#4))))))

	(year-to-days (lambda (year)
		(difference
			(plus
				(plus
					(times year '#365)
					(quotient year '#4))
				(quotient year '#400))
			(quotient year '#100))))

	(month-to-days (lambda (year month)
		(cond ((logand (leap-year-p year) (greaterp month '#2))
				(incr (car (nth days-per-month month))))
			(t (car (nth days-per-month month)))))))

	(plus (year-to-days (decr (year-of date)))
		(plus (month-to-days (year-of date) (month-of date))
			(day-of date)))))
