; ArrowLISP Extended Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Convert a rational number to readable fixed-point
; representation. Note that the resulting symbol is
; of little use, since ArrowLISP has no fixed-point
; arithmetic functions.
; --
; BUG: this function always returns '#0,0 for X=0.

; ---args---
; X - rational number to convert
; P - precision (number of fractional digits)

; ---keywords---
; FIXED-POINT, rational numbers, fixed-point numbers
; decimal-point numbers, rational arithmetics

; ---example---
; :l lib/rsquareroot.l
; (fixed-point (rational (rsquareroot '#2 '#3)) '#3)

:c lib/times.l
:c lib/expt.l
:c lib/numerator.l
:c lib/denominator.l
:c lib/quotient.l
:c lib/length.l
:c lib/append.l
:c lib/nth.l

; ---code---
(defun fixed-point (x p)
	(label ((fp-quotient (lambda (x p)
		(quotient
			(times (numerator x) (expt '#10 p))
			(denominator x))))
	(int-part (quotient (numerator x) (denominator x)))
	(int-len (cond ((zerop int-part) '#0) (t (length int-part))))
	(quot (fp-quotient x p))
	(append3 (lambda (a b c) (append a (append b c)))))
	(append3 int-part '#, (nth quot (incr int-len)))))

