; ArrowLISP Extended Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Retrieve a binding from an association tree.
; An association tree (atree) is an ordered binary
; tree containing (symbol.value) pairs. Each node
; of the tree has the format
; -
; (left (symbol.value) right)
; -
; where left and right are either nodes or NIL.

; ---args---
; X - key to find
; A - atree
; P - predicate defining order of tree

; ---keywords---
; TREE-ASSOC function, association lists, association trees,
; trees, ordered trees

; ---see-also---
; assoc, tree-insert, tree-balance, tree-flatten, tree-sort

; ---example---
; :l lib/lessp.l
; (define tree
;   '(((() (#1 1) (() (#2 2) ())) (#3 3) (() (#4 4) ())) (#5 5) ()))
; (tree-assoc '#1 tree lessp)

:c lib/equal.l
:c lib/caar.l
;c lib/cadr.l
:c lib/caaar.l
;c lib/caddr.l

; ---code---
(defun tree-assoc (x a p)
	(label ((tassoc (lambda (a)
		(cond ((null a) nil)
			((equal (caadr a) x) (cadr a))
			((p x (caadr a)) (tassoc (car a)))
			(t (tassoc (caddr a)))))))
	(tassoc a)))

