; ArrowLISP Extended Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Convert an ordered list into a balanced tree.

; ---args---
; X - list
; P - predicate defining order

; ---keywords---
; TREE-BALANCE function, tree, balanced tree, ordered trees,
; sorting, ordering

; ---see-also---
; tree-insert, tree-sort, tree-flatten, tree-assoc

; ---example---
; :l lib/lessp.l
; (tree-balance '(#1 #2 #3 #4 #5 #6 #7) lessp)

:c lib/reverse.l
:c lib/caar.l
;c lib/cadr.l
;c lib/cddr.l
:c ext/tree-insert.l

; ---code---
(defun tree-balance (x p)
	; Traverse the list using divide and conquer:
	; first insert the middle element, then the
	; left sub-list and finally the right sub-list.
	(label ((divide-list (lambda (x rx tree)
		(cond ((null x) tree)
			((eq (car x) (car rx))
				(divide-list (cdr x) (reverse (cdr x))
					(divide-list (reverse (cdr rx)) (cdr rx)
						(tree-insert (car x) tree p))))
			((eq (car x) (cadr rx))
				(divide-list (cddr x) (reverse (cddr x))
					(divide-list (reverse (cdr rx)) (cdr rx)
						(tree-insert (car rx) tree p))))
			(t (divide-list (cdr x) (cdr rx) tree))))))
	(divide-list x (reverse x) nil)))

