; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Copy an S-expression. Of course, this function
; is never required in pure LISP programs, since
; lists are never altered.
; -
; This function serves as a model for other
; functions that have to traverse list or tree
; structures.

; ---args---
; X - expression to copy

; ---keywords---
; COPY function, copy, traverse, lists, trees

; ---see-also---
; equal, subst, count

; ---example---
; (copy '((a (b.c)) ((d.e) f)))

:c lib/caar.l
;c lib/cadr.l
;c lib/cddr.l
:c lib/append.l
:c lib/logand.l
:c lib/litatom.l

; ---model---
; (defun copy (x)
;	(cond ((atom x) x)
;		(t (cons (copy (car x)) (copy (cdr x))))))

; ---model---
; ; TAIL-CALL VERSION
; (defun copy (x)
;(label
;	; Make sure X is recognized as a part of an expression
;	((protect (lambda (x) (list x)))
;	; Retrieve the value of a protected expression
;	(unprotect (lambda (x) (car x)))
;	(cpy (lambda (x r s)
;		; CONS operator found?
;		(cond ((atom x)
;				; Stack empty?
;				(cond ((null s)
;						; Cons the top members of the R stack
;						(cons (cadr r) (car r)))
;					; Another CONS on top of the S stack?
;					((atom (car s))
;						; Pop CONS from S, cons top members of R stack
;						(cpy 'cons (cons (cons (cadr r) (car r)) (cddr r))
;							(cdr s)))
;					; Subtree on top of S stack
;					(t
;						; Pop subtree from S, cons top members of R
;						(cpy (protect (car (unprotect s)))
;							(cons (cons (cadr r) (car r)) (cddr r))
;							(cdr s)))))
;			; Leaf found
;			((atom (unprotect x))
;				; Stack empty? The tree consists of a single atom
;				(cond ((null s) (unprotect x))
;					; CONS operator on S stack?
;					((atom (car s))
;						; Push X to R, pop CONS from S
;						(cpy 'cons (cons (unprotect x) r) (cdr s)))
;					(t
;						; Else push X to R, pop subtree from S
;						(cpy (car s) (cons (unprotect x) r) (cdr s)))))
;			; Subtree found
;			(t
;				; PUSH CONS to S, push CDR of tree to S,
;				; advance to CAR of tree
;				(cpy (protect (car (unprotect x))) r
;					(append (list (protect (cdr (unprotect x))) 'cons)
;						s)))))))
;
;	(cpy (protect x) nil nil)))

; ---code---
; LINEAR-RECURSIVE VERSION
; X MAY NOT CONTAIN ANY IMPROPER LISTS
(defun copy (x)
	(label ((cpy (lambda (x r)
		(cond ((null x) (reverse r))
			((atom x) x)
			((logand (litatom (car x)) (litatom (cdr x))) x)
			(t (cpy (cdr x) (cons (cpy (car x) nil) r)))))))
	(cpy x nil)))
