; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Recursively count the atoms of a list.
; Members of all sublists are included.
; For instance,
; -
; (COUNT '(1(2(3)))) => '#3.

; ---args---
; X - list

; ---keywords---
; COUNT function, list, member, size

; ---see-also---
; length, depth, append, copy

; ---example---
; (count '(a b (c (d) e) ((f))))

:c lib/incr.l

; ---model---
; (defun count (x)
;	(cond ((null x) '#0)
;		((atom x) '#1)
;		(t (plus (count (car x)) (count (cdr x))))))

; ---code---
; TAIL-RECURSIVE VERSION
(defun count (x)
	(label ((cnt (lambda (x r s)
		(cond ((null x)
				(cond ((null s) r)
					(t (cnt (car s) r (cdr s)))))
			((atom x)
				(cnt nil (incr r) s))
			(t
				(cnt (car x) r (cons (cdr x) s)))))))
	(cnt x '#0 nil)))
