; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Subtract two natural numbers.

; ---args---
; A - natural number
; B - natural number to be subtracted

; ---keywords---
; DIFFERENCE function, difference, subtract, subtraction
; arithmetics, natural

; ---see-also---
; digits, idifference, rdifference, plus, quotient, remainder, times

; ---example---
; (difference '#100 '#45)

(define difference t)

:c lib/digits.l
:c lib/append.l
:c lib/reverse.l
:c lib/pred.l
:c lib/normalize.l

; ---code---
(defun difference (a b)
	(label

	; Compute difference of two digits with borrow
	; X Y - digits
	; CI CO - borrow (carry) in, out
	; RESULT - (difference . carry out)
	((ddiff (lambda (x y ci co)
		(cond ((eq y 0)
				(cond ((eq ci 1) (ddiff x 1 0 co))	; subtract carry in
					(t (cons x co))))
			((null (pred x))						; handle overflow
				(ddiff 9 (pred y) ci 1))
			(t (ddiff (pred x) (pred y) ci co)))))

	; Subtract two lists of decimal digits forming reverse
	; decimal numbers. The result is in normal order.
	; A B - reverse lists of digits
	; C - carry flag
	; R - result (normal order)
	(diff (lambda (a b c r)
		(cond ((null a)
				(cond ((null b)				; A=() ; B=()
						(cond ((eq c 0) r)	; carry not set? OK
							(t (bottom			; A<B is undefined
								'difference-negative))))
					(t (bottom 'difference-negative))))
		((null b)							; A\=() ; B=() ; see above
			(cond ((eq c 0)					; carry not set?
					(append (reverse a) r))	; prepend rest of A
				(t (diff a '#1 0 r))))		; else subtract borrow
		(t (diff (cdr a) (cdr b)			; advance to next pair of digits
			(cdr (ddiff (car a) (car b) c 0))	; compute borrow
			(cons (car (ddiff (car a)			; prepend result digit
					(car b) c 0))
				r)))))))
	
	(normalize (diff (reverse a) (reverse b) 0 nil))))
