; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Divide two natural numbers, giving a
; result of the form (quotient remainder).

; ---args---
; A - natural number (dividend)
; B - natural number (divisor)

; ---keywords---
; DIVIDE function, quotient, modulus, division, divide
; arithmetics, natural

; ---see-also---
; digits, idivide, rquotient, quotient, remainder, plus
; difference, times

; ---example---
; (divide '#11 '#3)

(define divide t)

:c lib/digits.l
:c lib/normalize.l
:c lib/plus.l
:c lib/lessp.l
:c lib/equal.l
:c lib/difference.l
:c lib/incr.l
:c lib/decr.l
:c lib/zerop.l
:c lib/reverse.l
:c lib/append.l

; ---code---
(defun divide (a b)
	(label

	; Equalize the divisor by shifting it to the left
	; (multiplying it by 10) until it has the same number
	; of digits as the dividend.
	; A - list of digits (dividend)
	; B - reverse list of digits (divisor)
	; R - result of the form (new divisor . one I per shift)
	((eql (lambda (a b r)
		(cond ((null a)								; end of dividend?
				(cons (reverse (car r)) (cdr r)))	; reverse the divisor
			((null b) 						; end of divisor?
				(eql (cdr a) nil			; go to next digit
					(cons (cons 0 (car r))	; prepend 0 to divisor
						(cons 'i (cdr r)))))	; count shift
			(t (eql (cdr a) (cdr b)			; default: copy a digit
				(cons (cons (car b) (car r)) (cdr r)))))))

	; Divide two lists of digits which have a quotient
	; less than 10.
	; A - dividend
	; B - divisor
	; R - result of the form (A/B*B . A/B)
	(div10 (lambda (a b r)
		(cond ((lessp (car r) a)
				(div10 a b (cons (plus (car r) b) (incr (cdr r)))))
			((equal (car r) a) r)
			(t (cons (difference (car r) b) (decr (cdr r)))))))

	; X / 10
	(d10 (lambda (x) (reverse (cdr (reverse x)))))

	; X * 10
	(x10 (lambda (x) (append x '#0)))

	; Divide two lists of digits
	; A - list of digits (dividend)
	; B - divisor of the form (EQL A B ...)
	; R - result (list of digits in normal order)
	(div (lambda (a b r)
		(cond ((null (cdr b)) (cons (normalize r) (cons a nil)))
			(t (label ((quot (div10 a (car b) '(#0.#0))))
				(div (difference a (car quot))
					(cons (d10 (car b)) (cddr b))
					(plus (x10 r) (cdr quot)))))))))
	
	(cond ((zerop b) (bottom 'divide-by-zero))
		((lessp a b) (list '#0 a))
		(t (div a (eql a b '(() i)) '#0)))))
