; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Equalize two rational numbers by giving them
; equivalent denominators. Return a list containing
; the equalized numbers.

; ---args---
; A - rational number
; B - rational number

; ---keywords---
; EQUALIZE function, numerator, denominator, arithmetics, rational

; ---see-also---
; digits, rplus, rdifference, normalize

; ---example---
; (equalize '(/ #2 #3) '(/ #4 #5))

; PRE-LOAD TO AVOID LOAD BUFFER OVERFLOW
:c lib/normalize.l

:c lib/digits.l
:c lib/numerator.l
:c lib/denominator.l
:c lib/iquotient.l
:c lib/itimes.l
:c lib/gcd.l
:c lib/onep.l

; ---code---
(defun equalize (a b)
	(label ((num-a (numerator a))
		(num-b (numerator b))
		(denum-a (denominator a))
		(denum-b (denominator b))
		(cd (gcd denum-a denum-b)))
			(cond ((onep cd) (list
					(list '/
						(itimes num-a denum-b)
						(itimes denum-a denum-b))
					(list '/
						(itimes num-b denum-a)
						(itimes denum-b denum-a))))
				(t (list
					(list '/
						(iquotient (itimes num-a denum-b) cd)
						(iquotient (itimes denum-a denum-b) cd))
					(list '/
						(iquotient (itimes num-b denum-a) cd)
						(iquotient (itimes denum-b denum-a) cd)))))))

