; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Compute the X raised to the power of Y.
; Both X and Y must be integer numbers.
; Y must be positive or zero.

; ---args---
; X - integer number (base)
; Y - positive integer number (exponent)

; ---keywords---
; IEXPT function, base, exponent, power

; ---see-also---
; digits, expt, rexpt, gcd, fac

; ---example---
; (iexpt '#-3 '#9)

(define iexpt t)

:c lib/digits.l
:c lib/zerop.l
:c lib/evenp.l
:c lib/itimes.l
:c lib/quotient.l
:c lib/decr.l
:c lib/natural.l

; ---model---
; (defun iexpt (x y)
; 	(label ((exp (lambda (x y r)
; 		(cond ((zerop y) r)
; 			(t (exp x (decr y) (itimes x r)))))))
; 	(exp (integer x) (natural y) '#1)))

; ---code---
(defun iexpt (x y)
	(label ((square (lambda (x) (itimes x x)))
	(exp (lambda (x y)
		(cond ((zerop y) '#1)
			((evenp y)
				(square (exp x (quotient y '#2))))
			(t (itimes x (square (exp x (quotient y '#2)))))))))
	(exp x (natural y))))

