; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Check if two integer numbers are in ascending
; order. Return *T*, if A<B and otherwise *F*.

; ---args---
; A - integer number
; B - integer number

; ---keywords---
; ILESSP function, less than, ordering, relation
; comparison, predicate, integer

; ---see-also---
; digits, inlessp, lessp, rlessp, igreaterp, equal

; ---example---
; (ilessp '#-123 '#0)

:c lib/lessp.l
:c lib/integer.l

; ---code---
(defun ilessp (a b)
	(label

	; Check if A has less members than B
	((shorter (lambda (a b)
		(cond ((null a)
				(cond ((null b) *f*)
					(t *t*)))
			((null b) *f*)
			(t (shorter (cdr a) (cdr b))))))

	; Check if A has more members than B
	(longer (lambda (a b) (shorter b a)))

	; Handle signs and lengths
	(compare (lambda (a b)
		(cond ((eq (car a) '-)
			; A<0, B>=0 => A<B
			(cond ((eq (car b) '+) *t*)
				; A<0, B<0, A shorter than B => A>B
				((shorter a b) *f*)
				; A<0, B<0, A longer than B => A<B
				((longer a b) *t*)
				; last resort: compare digits
				(t (lessp (cdr b) (cdr a)))))
		; A>=0, B<0 => A>B
		((eq (car b) '-) *f*)
		; A>=0, B>=0, A shorter than B => A<B
		((shorter a b) *t*)
		; A>=0, B>=0, A longer than B => A>B
		((longer a b) *f*)
		; last resort: compare digits
		(t (lessp (cdr a) (cdr b)))))))

	(compare (integer a) (integer b))))
