; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Check if two natural numbers are in ascending
; order. Return *T*, if A<B and otherwise *F*.

; ---args---
; A - natural number
; B - natural number

; ---keywords---
; LESSP function, less than, ordering, relation
; comparison, predicate, natural

; ---see-also---
; digits, nlessp, ilessp, rlessp, greaterp, equal

; ---example---
; (lessp '#5 '#7)

:c lib/logand.l
:c lib/pred.l
:c lib/reverse.l

; ---code---
(defun lessp (a b)
	(label

	; Check the ordering of two digits
	; A - digit
	; B - digit
	; Result: A<B-> *T*; *F*
	((dlt (lambda (a b)
		(cond ((eq a b) *f*)
			((eq a 0) *t*)
			((eq b 0) *f*)
			(t (dlt (pred a) (pred b))))))

	; Check the inverse ordering of two digits
	; A - digit
	; B - digit
	; Result: A>B-> *T*; *F*
	(dgt (lambda (a b) (dlt b a)))
	
	; Check ordering.
	; A - reverse list of digits
	; B - reverse list of digits
	; R - result: member of {*T*,*F*}
	(ltp (lambda (a b r)
		(cond ((logand (null a) (null b)) r)	; all compared
			((null a) *t*)						; length[a] < length[b]
			((null b) *f*)						; length[a] > length[b]
			(t (ltp (cdr a) (cdr b)
				(cond ((dlt (car a) (car b)) *t*)	; default:
					((dgt (car a) (car b)) *f*)	; compare digits
					(t r))))))))

	(ltp (reverse a) (reverse b) *f*)))
