; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Normalize a number. Remove leading zeroes,
; reduce rational numbers, remove leading
; plus signs.

; ---args---
; X - number

; ---keywords---
; NORMALIZE function, normalize, normal form, canonical number
; GCD, reducing, numbers, digits, decimal, integer, natural,
; rational

; ---see-also---
; digits, difference, divide, equalize

; ---example---
; (normalize '(/ #+006 #4))

(define normalize t)

:c lib/digits.l
:c lib/caar.l
;c lib/cdar.l
:c lib/caaar.l
:c lib/caadr.l
:c lib/cdadr.l
:c lib/gcd.l
:c lib/onep.l
:c lib/zerop.l
:c lib/iquotient.l
:c lib/numerator.l
:c lib/denominator.l
:c lib/minus.l

; ---code---
(defun normalize (x)
	(label

	; remove leading zeroes from natural number
	((norm (lambda (x)
	(cond ((null (cdr x)) x)
		((eq (car x) 0)
			(norm (cdr x)))
		(t x))))

	; reduce rational number to least denominator
	(reduce (lambda (a b)
		(label ((cd (cond ((zerop a) nil) (t (gcd a b)))))
			(cond ((null cd) 
					(list '/ (normalize a) (normalize b)))
				((onep cd)
					(list '/ (normalize a) (normalize b)))
				(t (list '/
					(normalize (iquotient a cd))
					(normalize (iquotient b cd))))))))

	; Normalize sign of rational number by moving sign
	; to numerator
	(norm-sign (lambda (x)
		(cond ((eq (car (numerator x)) '-)
				(cond ((eq (car (denominator x)) '-)
						(list '/ (cdr (numerator x))
							(cdr (denominator x))))
					(t x)))
			((eq (car (denominator x)) '-)
				(list '/ (minus (numerator x))
					(cdr (denominator x))))
			(t x))))

	; convert rational with denominator of 1 to integer
	(decay (lambda (x)
		(cond ((onep (denominator x)) (numerator x))
			(t x)))))

	(cond ((eq (car x) '+)
			(norm (cdr x)))
		((eq (car x) '-)
			(cons '- (norm (cdr x))))
		((eq (car x) '/)
			(decay (norm-sign (reduce (numerator x) (denominator x)))))
		(t (norm x)))))
