; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Generate permutations of a set.

; ---args---
; X - set

; ---keywords---
; PERMUTE, permutation, combination

; ---see-also---
; unique, member

; ---example---
; (permute '#abc)

:c lib/caar.l
;l lib/cddr.l
:c lib/append.l
:c lib/reverse.l

; ---code---
(defun permute (x)
	(label

	; rotate a list by moving the first element to the tail
	((rotate (lambda (x)
		(append (cdr x) (list (car x)))))

	; distribute A over B by consing A to each member of B
	(dist (lambda (a b)
		(cond ((null b) nil)
			(t (cons (cons a (car b)) (dist a (cdr b)))))))

	; create permutations of each member L of X by
	; distributing (CAR L) over all permutations of (CDR L)
	(permlist (lambda (x r)
		(cond ((null x) r)
			(t (permlist (cdr x)
				(append (dist (caar x) (perm (cdar x) (cdar x) nil)) r))))))

	; create a list of all possible rotations of a list
	; and pass it to PERMLIST
	(perm (lambda (x n r)
		(cond ((null n) (permlist r nil))
			((null (cdr x)) x)
			((null (cddr x)) (list x (reverse x)))
			(t (perm (rotate x) (cdr n) (cons x r)))))))

	(perm x x nil)))
