; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Add two natural numbers.

; ---args---
; A - natural number
; B - natural number

; ---keywords---
; PLUS function, plus, sum, addition, arithmetics, natural

; ---see-also---
; digits, iplus, rplus, difference, quotient, remainder, times

; ---example---
; (plus '#5 '#7)

(define plus t)

:c lib/digits.l
:c lib/reverse.l
:c lib/pred.l
:c lib/succ.l

; ---code---
(defun plus (a b)
	(label

	; Compute the sum of two digits and a carry flag.
	; X Y - digits
	; CI CO - carry in, out
	; RESULT - (sum . carry out)
	((dsum (lambda (x y ci co)
		(cond ((eq x 0)
				(cond ((eq ci 1) (dsum 1 y 0 co))	; add carry flag
					(t (cons y co))))
			((null (succ y))						; handle overflow
				(dsum (pred x) 0 ci 1))
			(t (dsum (pred x) (succ y) ci co)))))

	; Add two lists of decimal digits forming reverse
	; decimal numbers. The result is in normal order.
	; A B - reverse lists of digits
	; C - carry flag
	; R - result (normal order)
	(add (lambda (a b c r)
		(cond ((null a)
				(cond ((null b)			; A=() ; B=()
						(cond ((eq c 0) r)		; carry not set? return R
							(t (cons 1 r))))	; else prepend overflow
					(t (add nil (cdr b)		; B\=(): keep adding leading zeroes
						(cdr (dsum 0 (car b) c 0))
						(cons (car (dsum 0 (car b) c 0)) r)))))
		((null b)
			(add (cdr a) nil			; B=() ; A\=()
				(cdr (dsum (car a) 0 c 0))
				(cons (car (dsum (car a) 0 c 0)) r)))
		(t (add (cdr a) (cdr b)			; default: advance to next two digits
			(cdr (dsum (car a) (car b) c 0))
			(cons (car (dsum (car a) (car b) c 0)) r)))))))

	(add (reverse a) (reverse b) 0 nil)))
