; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Compute the X raised to the power of Y.
; X may be a natural, integer, or rational number.
; Y must be an integer.

; ---args---
; X - number (base)
; Y - number (exponent)

; ---keywords---
; REXPT function, base, exponent, power

; ---see-also---
; digits, expt, iexpt, gcd, fac

; ---example---
; (rexpt '#1 '#-5)

(define rexpt t)

:c lib/digits.l
:c lib/minusp.l
:c lib/abs.l
:c lib/rquotient.l
:c lib/zerop.l
:c lib/evenp.l
:c lib/rtimes.l
:c lib/quotient.l
:c lib/decr.l
:c lib/integer.l

; ---model---
; (defun rexpt (x y)
;	(label ((exp (lambda (x y r)
;		(cond ((zerop y) r)
;			(t (exp x (decr y) (rtimes x r)))))))
;	(cond ((minusp y)
;			(exp (rquotient '#1 (rational x)) (abs y) '#+1))
;		(t (exp (rational x) (integer y) '#1)))))

; ---code---
(defun rexpt (x y)
	(label ((xr
		(cond ((minusp y)
				(rquotient '#1 (rational x)))
			(t (rational x))))
	(square (lambda (x) (rtimes x x)))
	(exp (lambda (x y)
		(cond ((zerop y) '#1)
			((evenp y)
				(square (exp x (quotient y '#2))))
			(t (rtimes x (square (exp x (quotient y '#2)))))))))
	(exp xr (abs (integer y)))))
