; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Compute the square root of a number. This function
; returns a rational result, which may be inexact.

; ---args---
; X - square of root to extract
; PRECISION - maximum error (1/10^PRECISION)

; ---keywords---
; RSQUAREROOT function, root, square, square root
; arithmetics, rational

; ---see-also---
; digits, squareroot, rexpt, rquotient, rtimes

; ---example---
; (rsquareroot '#2 '#1)

:c lib/equal.l
:c lib/expt.l
:c lib/rtimes.l
:c lib/rquotient.l
:c lib/rplus.l
:c lib/abs.l
:c lib/rlessp.l
:c lib/rdifference.l
:c lib/squareroot.l

; ---code---
(defun rsquareroot (square precision)
	(label ((epsilon
		(list '/ '#1 (expt '#10 (natural precision))))
	(sqr (lambda (x)
		(cond ((rlessp
					(abs (rdifference (rtimes x x) square))
					epsilon)
				x)
			(t (sqr
				(rquotient
					(rplus x (rquotient square x))
					'#2)))))))
	(sqr (squareroot square))))
