; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Compute the square root of a natural number.
; This function returns a natural result, which
; may be inexact.

; ---args---
; X - square of root to extract

; ---keywords---
; SQUAREROOT function, root, square, square root
; arithmetics, natural

; ---see-also---
; digits, expt, times, quotient

; ---example---
; (squareroot '#144)

:c lib/equal.l
:c lib/minusp.l
:c lib/times.l
:c lib/quotient.l
:c lib/plus.l
:c lib/greaterp.l
:c lib/ngreaterp.l
:c lib/decr.l
:c lib/incr.l

; ---model---
; (defun squareroot (square)
;	(label ((sqr (lambda (x last)
;		(cond ((logand (ngreaterp (times x x) square)
;					(greaterp (times (incr x) (incr x)) square))
;				x)
;			(t (sqr
;				(quotient
;					(plus x (quotient square x))
;					'#2) x))))))
;	(sqr square '#0)))

; ---code---
(defun squareroot (square)
	(label ((sqr (lambda (x last)
		(cond ((equal last x) x)
			((equal last (incr x))
				(cond ((greaterp (times x x) square) (decr x))
					(t x)))
			(t (sqr
				(quotient
					(plus x (quotient square x))
					'#2) x))))))
	(cond ((minusp square) (bottom (list 'squareroot square)))
		(t (sqr square '#0)))))
