; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Substitute variables in S-expressions.
; For a description of association lists (alists)
; see ASSOC.

; ---args---
; A - alist with bindings
; X - S-expression to process

; ---keywords---
; SUBLIS function, substitution, substitute variables
; association lists

; ---see-also---
; assoc, sassoc, subst

; ---example---
; (sublis '((1.i) (2.ii) (3.iii)) '(1 + 2 - 3))

:c lib/assoc.l
:c lib/reverse.l
:c lib/caar.l
;c lib/cdar.l
:c lib/append.l
:c lib/litatom.l

; ---model---
; (defun sublis (a x)
;	(cond ((null x) nil)
;		((atom x)
;			(cond ((null (assoc x a)) x) (t (cdr (assoc x a)))))
;		(t (cons (sublis a (car x)) (sublis a (cdr x))))))

; ---model---
; ; TAIL-CALL (BUT WAY TOO COMPLEX) VERSION
; ; FOR A DETAILED EXPLANATION, SEE 'COPY'
; (defun sublis (a x)
;	(label
;	((protect (lambda (x) (list x)))
;	(unprotect (lambda (x) (car x)))
;	(sla (lambda (x a)
;		(cond ((null a) x)
;			((eq (caar a) x) (cdar a))
;			(t (sla x (cdr a))))))
;	(sls (lambda (x r s)
;		(cond ((atom x)
;				(cond ((null s)
;						(cons (cadr r) (car r)))
;					((atom (car s))
;						(sls 'cons (cons (cons (cadr r) (car r))
;								(cddr r))
;							(cdr s)))
;					(t (sls (protect (car (unprotect s)))
;							(cons (cons (cadr r) (car r)) (cddr r))
;							(cdr s)))))
;			((atom (unprotect x))
;				(cond ((null s) (sla (unprotect x) a))
;					((atom (car s))
;						(sls 'cons (cons (sla (unprotect x) a) r)
;							(cdr s)))
;					(t (sls (car s) (cons (sla (unprotect x) a) r)
;						(cdr s)))))
;			(t (sls (protect (car (unprotect x))) r
;					(append (list (protect (cdr (unprotect x))) 'cons)
;						s)))))))
;	(sls (protect x) nil nil)))

; ---code---
; X MAY NOT CONTAIN ANY IMPROPER LISTS
(defun sublis (a x)
	(label ((sla (lambda (x a)
		(cond ((null a) x)
			((eq (caar a) x) (cdar a))
			(t (sla x (cdr a))))))
	(sls (lambda (x r)
		(cond ((null x) (reverse r))
			((atom x) (sla x a))
			((logand (litatom (car x)) (litatom (cdr x)))
				(cons (sls (car x) nil) (sls (cdr x) nil)))
			(t (sls (cdr x) (cons (sls (car x) nil) r)))))))
	(sls x nil)))
