; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Substitute X for each occurence of Y in Z.
; X, Y, Z are S-expressions.

; ---args---
; X - expression to replace Y with
; Y - expression to find
; Z - expression to process

; ---keywords---
; SUBST function, substitution, substitute, lists

; ---see-also---
; member, equal, sublis

; ---example---
; (subst 'y1 'x2 '(lambda (x1 x2) (x1 x2)))

:c lib/equal.l
:c lib/append.l
:c lib/reverse.l
:c lib/caar.l
;c lib/cdar.l
:c lib/litatom.l

; ---model---
; (defun subst (x y z)
;	(cond ((equal y z) x)
;		((atom z) z)
;		(t (cons (subst x y (car z)) (subst x y (cdr z))))))

; ---model---
; ; TAIL-CALL (BUT WAY TOO COMPLEX) VERSION
; ; SEE 'COPY' FOR DETAILS.
; (defun subst (x y z)
;	(label
;	((protect (lambda (z) (list z)))
;	(unprotect (lambda (z) (car z)))
;	(sbs (lambda (z r s)
;		(cond ((atom z)
;				(cond ((null s)
;						(cons (cadr r) (car r)))
;					((atom (car s))
;						(sbs 'cons (cons (cons (cadr r) (car r))
;								(cddr r))
;							(cdr s)))
;					(t (sbs (protect (car (unprotect s)))
;							(cons (cons (cadr r) (car r)) (cddr r))
;							(cdr s)))))
;			((equal y (unprotect z))
;				(cond ((null s) x)
;					((atom (car s))
;						(sbs 'cons (cons x r) (cdr s)))
;					(t (sbs (car s) (cons x r) (cdr s)))))
;			((atom (unprotect z))
;				(cond ((null s) (unprotect z))
;					((atom (car s))
;						(sbs 'cons (cons (unprotect z) r) (cdr s)))
;					(t (sbs (car s) (cons (unprotect z) r) (cdr s)))))
;			(t (sbs (protect (car (unprotect z))) r
;					(append (list (protect (cdr (unprotect z))) 'cons)
;						s)))))))
;	(sbs (protect z) nil nil)))

; ---code---
; LINEAR-RECURSIVE VERSION
; Z MAY NOT CONTAIN ANY IMPROPER LISTS
(defun subst (x y z)
	(label ((sbs (lambda (z r)
		(cond ((null z) (reverse r))
			((equal y z) x)
			((atom z) z)
			((logand (litatom (car z)) (litatom (cdr z)))
				(cons (sbs (car z) nil) (sbs (cdr z) nil)))
			(t (sbs (cdr z) (cons (sbs (car z) nil) r)))))))
	(sbs z nil)))
