; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Compute the product of two natural numbers.

; ---args---
; A - natural number (factor 1)
; B - natural number (factor 2)

; ---keywords---
; TIMES function, times, multiply, multiplication, product
; arithmetics, natural

; ---see-also---
; digits, itimes, rtimes, plus, difference, quotient, remainder

; ---example---
; (times '#4 '#5)

(define times t)

:c lib/digits.l
:c lib/plus.l
:c lib/pred.l
:c lib/append.l
:c lib/reverse.l
:c lib/zerop.l

; ---code---
(defun times (a b)
	(label

	; X*10 where X=/=0
	((x10 (lambda (x) (append x '#0)))

	; Add A to R B times.
	; A,R are numbers, B is a decimal digit.
	(addn (lambda (a b r)
		(cond ((eq b 0) r)
			(t (addn a (pred b) (plus a r))))))

	; R=A*B
	; B is in reverse order.
	(tms (lambda (a b r)
		(cond ((null b) r)
			(t (tms (x10 a) (cdr b) (addn a (car b) r)))))))

	; avoid leading zeroes in result
	(cond ((zerop a) '#0)
		(t (tms a (reverse b) '#0)))))
