; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Compute the union of two sets.
; Sets are represented by lists
; of unique members.

; ---args---
; A - set
; B - set

; ---keywords---
; UNION function, union, sets, set theory

; ---see-also---
; intersection, member, remove, unique, any

; ---example---
; (union '(alpha beta gamma) '(beta delta))

:c lib/member.l
:c lib/append.l
:c lib/reverse.l
:c lib/unique.l

; ---model---
; (defun union (a b)
;	(cond ((null a) b)
;		((member (car a) b) (union (cdr a) b))
;		(t (cons (car a) (union (cdr a) b)))))

; ---model---
; ; TAIL-RECURSIVE VERSION
; (defun union (a b)
;	(label ((uon (lambda (a b r)
;		(cond ((null a) (append (reverse r) b))
;			((member (car a) b) (uon (cdr a) b r))
;			(t (uon (cdr a) b (cons (car a) r)))))))
;	(uon a b nil)))

; ---code---
(defun union (a b) (unique (append a b)))
