; ArrowLISP Library
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Generate some entropy by un-sorting a list of
; natural numbers. The order of the list A is
; changed in a hard to predict way by extracting
; the (N mod length[A])'th element from A and
; consing it to B. Each extracted element is
; removed from A. When A is empty, the result
; is in B.

; ---args---
; A - list to unsort
; SEED - first element to arrange, where 0<SEED<=length[A]

; ---keywords---
; UNSORT function, entropy, order, sorting, unsorting, random.

; ---see-also---
; sort, insert, orderedp

; ---example---
; :l lib/iota.l
; (unsort (iota '#10) '#5)

:c lib/digits.l
:c lib/length.l
:c lib/decr.l
:c lib/zerop.l
:c lib/remainder.l

; ---code---
(defun unsort(a seed)
	(label

	; Extract the n'th element of a list
	((nth (lambda (a n)
		(cond ((zerop n) (car a))
			(t (nth (cdr a) (decr n))))))

	; Generate a list with the n'th element removed
	(nonth (lambda (a n r)
		(cond ((zerop n) 
				(cond ((null a) (reverse r))
					(t (append (cdr a) (reverse r)))))
			(t (nonth (cdr a) (decr n) (cons (car a) r))))))

	; Unsort list A of length K by first extracting element N,
	; keep intermediate result in R
	(usrt (lambda (a n k r)
		(cond ((zerop k) (cons (car a) r))
			(t (usrt (nonth a n nil) (remainder (car a) k)
					(decr k) (cons (nth a n) r)))))))

	(usrt a seed (decr (length a)) nil)))
