; ArrowLISP Example Program
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Print the lyrics of "99 bottles of beer on the wall".

; ---keywords---
; BOTTLES, 99 bottles of beer, example programs

; ---example---
; (bottles)

; ---code---
; BOTTLES SONG IN PURE LISP
; COPYRIGHT (C) 2003 NILS M HOLM <NMH@T3X.ORG>
;
; THIS IS PURELY SYMBOLIC LISP.
; THERE ARE NO PRE-DEFINED NUMBERS, NOT EVEN DIGITS.
; THERE ARE NO STRINGS.
; THERE IS NO INPUT/OUTPUT.
; THERE ARE NO SIDE EFFECTS.
;
; THE BELOW EXPRESSION REDUCES TO A LIST CONTAINING
; SUBLISTS. EACH SUBLIST CONTAINS ONE VERSE OF THE SONG.
; EACH VERSE CONSISTS OF FOUR SUBLISTS. EACH SUBLIST OF
; A VERSE CONTAINS ONE SENTENCE.
; NUMBERS ARE REPRESENTED BY LISTS OF DIGITS.
;
; FOR EXAMPLE, THE FIRST VERSE OF THE SONG LOOKS LIKE
; THIS:
;
; (((9 9) BOTTLES OF BEER ON THE WALL)
;  ((9 9) BOTTLES OF BEER)
;  (TAKE ONE DOWN AND PASS IT AROUND)
;  ((9 8) BOTTLES OF BEER ON THE WALL))
;
; REQUIRED SUBROUTINES:
; LABEL, LAMBDA, QUOTE, CONS, CAR, CDR, EQ, NULL, COND
;
; HERE WE GO...

(define bottles (lambda ()
	(label

	; DEFINE SOME SENTENCES
	((beer-on-wall (quote (bottles of beer on the wall)))
	(beer (quote (bottles of beer)))
	(take-down (quote (take one down and pass it around)))

	; APPEND TWO LISTS
	(append (lambda (a b)
		(cond ((null a) b)
			(t (cons (car a) (append (cdr a) b))))))

	; APPEND FOUR LISTS
	(append4 (lambda (a b c d)
		(append a (append b (append c d)))))

	; FIND THE PREDECESSOR OF A DIGIT, WHERE (PRED (QUOTE 0)) => NIL
	(pred (lambda (n)
		(cond
			((eq n (quote 1)) (quote 0))
			((eq n (quote 2)) (quote 1))
			((eq n (quote 3)) (quote 2))
			((eq n (quote 4)) (quote 3))
			((eq n (quote 5)) (quote 4))
			((eq n (quote 6)) (quote 5))
			((eq n (quote 7)) (quote 6))
			((eq n (quote 8)) (quote 7))
			((eq n (quote 9)) (quote 8))
			(t nil))))

	; THE CAR OF THE CDR OF A LIST
	(cadr (lambda (x) (car (cdr x))))

	; FORM A ONE-ELEMENT LIST
	(list (lambda (x) (cons x nil)))

	; DECREMENT A TWO-DIGIT NATURAL NUMBER
	(decrement (lambda (n)
		(cond ((null (pred (cadr n)))
				(cons (pred (car n)) (list (quote 9))))
			(t (cons (car n) (list (pred (cadr n))))))))

	; CHECK IF N EQUALS (QUOTE (0 0))
	(zerop (lambda (n)
		(cond ((eq (car n) (quote 0))
				(cond ((eq (cadr n) (quote 0)) t)
						(t nil)))
			(t nil))))

	; FORM A VERSE
	(verse (lambda (n)
		(append4
			(list (append (list n) beer-on-wall))
			(list (append (list n) beer))
			(list take-down)
			(list (append (list (decrement n)) beer-on-wall)))))

	; COLLECT THE VERSES IN A TAIL-RECURSIVE FUNCTION
	(count-bottles (lambda (n lyrics)
		(cond ((zerop n) lyrics)
			(t (count-bottles (decrement n)
				(append lyrics (list (verse n)))))))))

	(count-bottles (quote (9 9)) nil))))

