; ArrowLISP Example Program
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Solve the N-queens problem using backtracking.
; The number of queens (N) is specified as a list
; of digits.
; --
; NOTICE: this program is SLOW. It requires more
; than 1.9 million reductions to solve (QUEENS '#4).

; ---args---
; N - list of digits

; ---keywords---
; QUEENS function, queens, eight queens, N queens, N-queens, example

; ---example---
; (queens '#4)

:l lib/quotient.l
:l lib/remainder.l
:l lib/equal.l
:l lib/lessp.l
:l lib/difference.l
:l lib/times.l
:l lib/plus.l
:l lib/nequal.l
:l lib/incr.l
:l lib/decr.l

; ---code---
(defun queens (board-size)
	(label

	; Translate square number to column number
	((column (lambda (x) (quotient x board-size)))

	; Translate square number to row number
	(row (lambda (x) (remainder x board-size)))

	; Can X attack Y horizontally or vertically?
	(can-attack-hv-p (lambda (x y)
		(cond ((equal (row x) (row y)) *t*)
			(t (equal (column x) (column y))))))

	; Compute |X-Y|
	(abs-diff (lambda (x y)
		(cond ((lessp x y) (difference y x))
			(t (difference x y)))))

	; Can X attack Y diagonally?
	(can-attack-dia-p (lambda (x y)
		(equal (abs-diff (column x) (column y))
			(abs-diff (row x) (row y)))))

	; Can X attack Y?
	(can-attack-p (lambda (x y)
		(cond ((can-attack-hv-p x y) t)
			(t (can-attack-dia-p x y)))))

	; Test whether the square X is safe on a board
	; already containing the pieces listed in B
	(safe-place-p (lambda (x b)
		(cond ((null b) *t*)
			((can-attack-p (car b) x) *f*)
			(t (safe-place-p x (cdr b))))))

	; Compute the number of the first square
	; of the next column, where Q is any square
	; in the current column
	(next-column (lambda (q)
		(times (quotient (plus q board-size) board-size)
			board-size)))

	; Solve N Queens:
	; Q = next square to check
	; C = current column
	; B = board so far
	(n-queens (lambda (q c b)
		(cond ((equal c board-size) b)
			((nequal (column q) c)
				(cond ((null b) nil)
					(t (n-queens (incr (car b)) (decr c) (cdr b)))))
			((safe-place-p q b)
				(n-queens (next-column q) (incr c) (cons q b)))
			(t
				(n-queens (incr q) c b))))))

	(n-queens '#0 '#0 nil)))
