; ArrowLISP Example Program
; Copyright (C) 1998-2004 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; ---purpose---
; Convert a LABEL into an equivalent LAMBDA expression.
; --
; This program proves that any LABEL can be converted
; into an equivalent LAMBDA expression by automating
; this process.
; --
; Notice that
; --
; ((UNLABEL UNLABEL) UNLABEL)
; --
; breaks ArrowLISP, since it does not perform full
; beta-reduction.

; ---args---
; X - expression to convert

; ---keywords---
; LABEL, LAMBDA, scoping, example

; ---example---
; (unlabel '(label ((a '#5) (b '#7)) (plus a b)))

:l lib/reverse.l
:l lib/append.l
:l lib/caar.l
;l lib/cadr.l
:l lib/caaar.l
;l lib/caddr.l
;l lib/cadar.l

; ---code---
(defun unlabel(x)
	(label

	; Create list of symbols to bind in LABEL
	((symlist (lambda (x r)
		(cond ((null x) (reverse r))
			(t (symlist (cdr x) (cons (caar x) r))))))

	; Create list of (unlabeled) values to bind
	(vallist (lambda (x r)
		(cond ((null x) (reverse r))
			(t (vallist (cdr x) (cons (ulbl (cadar x)) r))))))

	; Extract (unlabeled) term of label
	(term (lambda (x) (ulbl (caddr x))))

	; Create lambda expression
	(makelambda (lambda (x)
		(append (list (list 'lambda (symlist (cadr x) nil) (term x)))
			(vallist (cadr x) nil))))

	; Traverse expression tree, replacing labels with lambda
	(ulbl (lambda (x)
		(cond ((atom x) x)
			((eq (car x) 'quote) x)
			((eq (car x) 'label) (makelambda x))
			(t (cons (ulbl (car x)) (ulbl (cdr x))))))))

(ulbl x)))
